@@ -22,7 +22,9 @@
 #include "libavutil/opt.h"
 #include "libavutil/pixdesc.h"
 #include "avfilter.h"
+#include "formats.h"
 #include "internal.h"
+#include "video.h"
 
 typedef struct WeaveContext {
     const AVClass *class;
@@ -30,7 +32,6 @@ typedef struct WeaveContext {
     int double_weave;
     int nb_planes;
     int planeheight[4];
-    int outheight[4];
     int linesize[4];
 
     AVFrame *prev;
@@ -80,9 +81,6 @@ static int config_props_output(AVFilterLink *outlink)
     s->planeheight[1] = s->planeheight[2] = AV_CEIL_RSHIFT(inlink->h, desc->log2_chroma_h);
     s->planeheight[0] = s->planeheight[3] = inlink->h;
 
-    s->outheight[1] = s->outheight[2] = AV_CEIL_RSHIFT(2*inlink->h, desc->log2_chroma_h);
-    s->outheight[0] = s->outheight[3] = 2*inlink->h;
-
     s->nb_planes = av_pix_fmt_count_planes(inlink->format);
 
     return 0;
@@ -108,20 +106,19 @@ static int weave_slice(AVFilterContext *ctx, void *arg, int jobnr, int nb_jobs)
         const int height = s->planeheight[i];
         const int start = (height * jobnr) / nb_jobs;
         const int end = (height * (jobnr+1)) / nb_jobs;
-        const int compensation = 2*end > s->outheight[i];
 
         av_image_copy_plane(out->data[i] + out->linesize[i] * field1 +
                             out->linesize[i] * start * 2,
                             out->linesize[i] * 2,
                             in->data[i] + start * in->linesize[i],
                             in->linesize[i],
-                            s->linesize[i], end - start - compensation * field1);
+                            s->linesize[i], end - start);
         av_image_copy_plane(out->data[i] + out->linesize[i] * field2 +
                             out->linesize[i] * start * 2,
                             out->linesize[i] * 2,
                             s->prev->data[i] + start * s->prev->linesize[i],
                             s->prev->linesize[i],
-                            s->linesize[i], end - start - compensation * field2);
+                            s->linesize[i], end - start);
     }
 
     return 0;
@@ -153,8 +150,17 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *in)
                       FFMIN(s->planeheight[1], ff_filter_get_nb_threads(ctx)));
 
     out->pts = s->double_weave ? s->prev->pts : in->pts / 2;
+#if FF_API_INTERLACED_FRAME
+FF_DISABLE_DEPRECATION_WARNINGS
     out->interlaced_frame = 1;
     out->top_field_first = !s->first_field;
+FF_ENABLE_DEPRECATION_WARNINGS
+#endif
+    out->flags |= AV_FRAME_FLAG_INTERLACED;
+    if (s->first_field)
+        out->flags &= ~AV_FRAME_FLAG_TOP_FIELD_FIRST;
+    else
+        out->flags |= AV_FRAME_FLAG_TOP_FIELD_FIRST;
 
     if (!s->double_weave)
         av_frame_free(&in);
